/*
 * alcd.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/channels.h>
#include "i18n.h"
#include "activylcd.h"

static const char *VERSION        = "0.0.2b";
static const char *DESCRIPTION    = "Activy300 LCD-Display Driver";
static const char *MAINMENUENTRY  = NULL;
aLCD *LCD;

class cAlcdStatus : public cStatus {
protected:
  virtual void ChannelSwitch(const cDevice *Device, int ChannelNumber);
  virtual void Recording(const cDevice *Device, const char *Name);
  virtual void Replaying(const cControl *Control, const char *Name);
  virtual void SetVolume(int Volume, bool mute);
  virtual void OsdProgramme(time_t PresentTime, const char *PresentTitle, const char *PresentSubtitle, time_t FollowingTime, const char *FollowingTitle, const char *FollowingSubtitle);
  };

void cAlcdStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
{
  switch (ChannelNumber)
    {
    case 0:
      break; // TODO: Indicator during Channel Switching
    default:
      if (Device->IsPrimaryDevice());
      {
	LCD->SetChannel(ChannelNumber);
	LCD->SetChannelName(Channels.GetByNumber(ChannelNumber)->Name());
      }
    }
}

void cAlcdStatus::Recording(const cDevice *Device, const char *Name)
{
  LCD->SetRecording(Device->CardIndex(), Name);
}

void cAlcdStatus::Replaying(const cControl *Control, const char *Name)
{
  LCD->ReplayControl((cControl *) Control,Name);
}

void cAlcdStatus::OsdProgramme(time_t PresentTime, const char *PresentTitle, const char *PresentSubtitle, time_t FollowingTime, const char *FollowingTitle, const char *FollowingSubtitle)
{
  LCD->SetProgramme(PresentTitle);
}

void cAlcdStatus::SetVolume(int Volume, bool mute)
{
  LCD->SetVolume(Volume,mute);
}

class cPluginAlcd : public cPlugin {
private:
  // Add any member variables or functions you may need here.
  cAlcdStatus *statusMonitor;

public:
  cPluginAlcd(void);
  virtual ~cPluginAlcd();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual void Housekeeping(void);
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdMenu *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  };

cPluginAlcd::cPluginAlcd(void)
{
  // Initialize any member varaiables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
statusMonitor = NULL;
}

cPluginAlcd::~cPluginAlcd()
{
  // Clean up after yourself!
  delete statusMonitor;
  delete LCD;
}

const char *cPluginAlcd::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return NULL;
}

bool cPluginAlcd::ProcessArgs(int argc, char *argv[])
{
  // Implement command line argument processing here if applicable.
  return true;
}

bool cPluginAlcd::Start(void)
{
  // Start any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  statusMonitor = new cAlcdStatus;
  LCD = new aLCD;
  LCD->Start();
  return true;
}

void cPluginAlcd::Housekeeping(void)
{
  // Perform any cleanup or other regular tasks.
}

cOsdMenu *cPluginAlcd::MainMenuAction(void)
{
  // Perform the action when selected from the main VDR menu.
  return NULL;
}

  int PowerButton = 0; // PowerButton off
  int WaitTime = 2;
  int RecordLED = 1; // Message LED
  int LEDBrightness = 0; // Brightness 0 == bright, 4 == dark...
  int LEDBrightnessOnExit = 4; 

class cMenuSetupAlcd : public cMenuSetupPage {
private:
  int newWaitTime;
  int newRecordLED;
  int newPowerButton;
  int newLEDBrightness;
  int newLEDBrightnessOnExit;
protected:
  virtual void Store(void);
public:
  cMenuSetupAlcd(void);
  };

cMenuSetupAlcd::cMenuSetupAlcd(void)
{
  static const char *RecLED[3];
  RecLED[0] = tr("off");
  RecLED[1] = "Message LED";
  RecLED[2] = "Basic LED";
  static const char *LEDBright[5];
  LEDBright[0] = tr("superbright");
  LEDBright[1] = tr("bright");
  LEDBright[2] = tr("semibright");
  LEDBright[3] = tr("dark");
  LEDBright[4] = tr("off");
  newWaitTime = WaitTime;
  newRecordLED = RecordLED;
  newPowerButton = PowerButton;
  newLEDBrightness = LEDBrightness;
  newLEDBrightnessOnExit = LEDBrightnessOnExit;
  Add(new cMenuEditBoolItem( tr("Power Button"), &newPowerButton, tr("off"), tr("on")));
  Add(new cMenuEditIntItem( tr("Wait time (s)"), &newWaitTime, 0, 60));
  Add(new cMenuEditStraItem( tr("Record LED"),  &newRecordLED, 3, RecLED)); 
  Add(new cMenuEditStraItem( tr("Display Brightness"), &newLEDBrightness, 4, LEDBright));
  Add(new cMenuEditStraItem( tr("Display Brightness on exit"), &newLEDBrightnessOnExit, 5, LEDBright));
}

void cMenuSetupAlcd::Store(void)
{
  SetupStore("PowerButton", PowerButton = newPowerButton);
  SetupStore("WaitTime", WaitTime = newWaitTime);
  SetupStore("RecordLED", RecordLED = newRecordLED);
  SetupStore("LEDBrightness", LEDBrightness = newLEDBrightness);
  SetupStore("LEDBrightnessOnExit", LEDBrightnessOnExit = newLEDBrightnessOnExit);
  LCD->SetLEDBrightness(LEDBrightness);
}

cMenuSetupPage *cPluginAlcd::SetupMenu(void)
{
  // Return a setup menu in case the plugin supports one.
  return new cMenuSetupAlcd;
}

bool cPluginAlcd::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  if      (!strcasecmp(Name, "PowerButton"))                   PowerButton = atoi(Value);
  else if (!strcasecmp(Name, "WaitTime"))                         WaitTime = atoi(Value);
  else if (!strcasecmp(Name, "RecordLED"))                       RecordLED = atoi(Value);
  else if (!strcasecmp(Name, "LEDBrightness"))               LEDBrightness = atoi(Value);
  else if (!strcasecmp(Name, "LEDBrightnessOnExit"))   LEDBrightnessOnExit = atoi(Value);
  else
     return false;
  return true;
               
}

VDRPLUGINCREATOR(cPluginAlcd); // Don't touch this!
